/*
 * Copyright (c) 2015 Cisco and/or its affiliates.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * physmem.h: virtual <-> physical memory mapping for VLIB buffers
 *
 * Copyright (c) 2008 Eliot Dresselhaus
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 *  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 *  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 *  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef included_vlib_physmem_h
#define included_vlib_physmem_h

typedef u8 vlib_physmem_region_index_t;

typedef struct
{
  vlib_physmem_region_index_t index;
  void *mem;
  uword size;
  int fd;
  u8 log2_page_size;
  u16 n_pages;
  u32 page_mask;

  void *heap;
  u32 flags;
#define VLIB_PHYSMEM_F_INIT_MHEAP		(1 << 0)
#define VLIB_PHYSMEM_F_HUGETLB			(1 << 1)
#define VLIB_PHYSMEM_F_SHARED			(1 << 2)

  u8 numa_node;
  u64 *page_table;
  u8 *name;
} vlib_physmem_region_t;



typedef struct
{
  u32 flags;
#define VLIB_PHYSMEM_MAIN_F_HAVE_PAGEMAP	(1 << 0)
#define VLIB_PHYSMEM_MAIN_F_HAVE_IOMMU		(1 << 1)
  vlib_physmem_region_t *regions;
} vlib_physmem_main_t;

extern vlib_physmem_main_t physmem_main;

#endif /* included_vlib_physmem_h */

/*
 * fd.io coding-style-patch-verification: ON
 *
 * Local Variables:
 * eval: (c-set-style "gnu")
 * End:
 */
